// ----------------------------------------------------------------------- //
// _vector.cpp	vector math stuff
//
// last edited:	2001-05-12 by Bjrn Paetzel
// 
// Copyright (C) 2001 by Bjrn Paetzel <kolrabi@gmx.de>
// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// includes

#include "math.h"

#include "_shared.h"
#include "_render.h"

// ----------------------------------------------------------------------- //

// ----------------------------------------------------------------------- //
// functions



// sets a vector to (0,0,0)

void _vector_clear
	(
		vec3_t v					// vector to clear
	)
{
	v[0] = v[1] = v[2] = 0;
} // _vector_clear




// get |v|

float _vector_length				// returns the lenght of a vector
	(
		const vec3_t v				// vector to measure
	) 
{
	return (float)sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]);	// sqrts are slow :(
} // _vector_length




// returns true if the distance betweeen two vectors is smaller than epsilon

int _vector_compare
	( 
		const vec3_t v1,			// vector 1
		const vec3_t v2				// vector 2
	) 
{
	int		i;
	
	for (i=0 ; i<3 ; i++)
		if (fabs(v1[i]-v2[i]) > EQUAL_EPSILON)
			return false;
			
	return true;
} // _vector_compare




// make the cross product

void _vector_cross
	( 
		const vec3_t v1,			// vector 1
		const vec3_t v2,			// vector 2
		vec3_t cross				// output vector
	) 
{
	cross[0] = v1[1]*v2[2] - v1[2]*v2[1];
	cross[1] = v1[2]*v2[0] - v1[0]*v2[2];
	cross[2] = v1[0]*v2[1] - v1[1]*v2[0];
} // _vector_cross




// dot product 

float _vector_dot					// return v1  v2
	( 
		vec3_t v1,					// vector 1
		vec3_t v2					// vector 2
	)
{
	return v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2];
} // _vector_dot




// out = v1 - v2

void _vector_subtract
	( 
		vec3_t v1,					// vector 1
		vec3_t v2,					// vector 2
		vec3_t out					// output vector
	)
{
	out[0] = v1[0]-v2[0];
	out[1] = v1[1]-v2[1];
	out[2] = v1[2]-v2[2];
} // _vector_subtract




// out = v1 + v2

void _vector_add
	( 
		vec3_t v1,					// vector 1
		vec3_t v2,					// vector 2
		vec3_t out					// output vector
	)
{
	out[0] = v1[0]+v2[0];
	out[1] = v1[1]+v2[1];
	out[2] = v1[2]+v2[2];
} // _vector_add




// out = in :)

void _vector_copy
	( 
		vec3_t in,					// source
		vec3_t out					// destination
	)
{
	out[0] = in[0];
	out[1] = in[1];
	out[2] = in[2];
} // _vector_copy




// out = v1 * scale

void _vector_scale
	( 
		vec3_t v,					// input vector
		vec_t scale,				// scale factor
		vec3_t out					// output vector
	)
{
	out[0] = v[0] * scale;
	out[1] = v[1] * scale;
	out[2] = v[2] * scale;
} // _vector_scale




// out = in / |in|

void _vector_normalize
	( 
		vec3_t in,					// input vector
		vec3_t out					// output vector
	) 
{
	if (out)
		_vector_scale( in, 1/_vector_length(in), out );
	else
		_vector_scale( in, 1/_vector_length(in), in );
} // _vector_normalize




// out = ( x, y, z )

void	_vector_set
	( 
		vec3_t out,					// output vector
		float x,					// coordinates
		float y, 
		float z 
	)
{
	out[0]=x; out[1]=y; out[2]=z;
}
// -------------------------------------------------------------------------



// -------------------------------------------------------------------------
// functions for vertices (flexVert_t) below

// vert = ( x, y, z, u, v, lu, lv )
void	_vertex_set(flexVert_t *vert, float x, float y, float z, float u, float v, float lu, float lv )
{
	vert->xyz[0] = x;		vert->xyz[1] = y;	vert->xyz[2] = z;
	vert->uv[0][0] = u;		vert->uv[0][1] = v;
	vert->uv[1][0] = lu;	vert->uv[1][1] = lv;
}

void	_vertex_copy(flexVert_t *in, flexVert_t *out )
{
	memcpy(out, in, sizeof(flexVert_t));
}

void	_vertex_rotatevertices( flexVert_t *buf, int count, float yaw, float pitch, float roll )
{
	flexVert_t	v;

	for (int i=0; i<count; i++)
	{
		memcpy(&v, &buf[i], sizeof(flexVert_t));

		_vector_set( v.xyz, 
			buf[i].xyz[0]*cos(yaw/180.0f*PI) - buf[i].xyz[2]*sin(yaw/180.0f*PI), 
			buf[i].xyz[1], 
			buf[i].xyz[0]*sin(yaw/180.0f*PI) + buf[i].xyz[2]*cos(yaw/180.0f*PI) );

		memcpy(&buf[i], &v, sizeof(flexVert_t));
	}
}

void	_vertex_translatevertices( flexVert_t *buf, int count, vec3_t v )
{
	for (int i=0; i<count; i++)
	{
		_vector_add( buf[i].xyz, v, buf[i].xyz );
	}
}

// -------------------------------------------------------------------------
// plane functions below

// create a plane from 3 points lying in it
plane_t	_plane_frompoints( vec3_t a, vec3_t b, vec3_t c )
{
	// need optimization ;)
	plane_t	p;
	vec3_t	ta,tb;

	_vector_subtract( b, a, ta );	
	_vector_subtract( c, a, tb );

	_vector_cross( ta, tb, p.n );
	_vector_normalize( p.n, p.n );

	p.d = -_vector_dot( p.n, a );

	return p;
}

// find the point in the line and in the plane
void	_plane_line_intersect( vec3_t a, vec3_t b, plane_t p, vec3_t out )
{
	float	aDot = _vector_dot(a,p.n);
	float	bDot = _vector_dot(b,p.n);
	float	scale = ( -p.d - aDot ) / ( bDot - aDot );

	vec3_t	d;

	_vector_subtract(b,a,d);
	_vector_scale(d,scale,d);
	_vector_add(d,a,out);
}

// ----------------------------------------------------------------------- //
